/*
	File:		GenericReadMe.c

*/

#include <Types.h>
#include <Memory.h>
#include <LowMem.h>
#include <Quickdraw.h>
#include <Files.h>
#include <Resources.h>
#include <TextUtils.h>
#include <TextEdit.h>
#include <ToolUtils.h>
#include <MixedMode.h>
#include <Events.h>
#include <Processes.h>
#include <assert.h>

#include "GenericReadMe.h"

// PROTOTYPES

static ShellErr	HandleMouseDownInImportantInfoPanel(EventRecord *inEvent, Boolean *wasHandled);
static OSErr	SetupInfoPanel( SInt16 inPrefRsrcID );

// GLOBALS

DialogPtr			gImportantInfoPanel = NULL;
PanelPtr			gHelpWindow 		= NULL;


//--------------------------------------------------------------------------------
//	InitializePluginModule
//--------------------------------------------------------------------------------

void InitializePluginModule(void* inPSTable, SInt32 inRefCon, Boolean inEnterAtBeginning)
{
#pragma unused (inEnterAtBeginning)

	ShellErr			err;
	
	EnterPlugin();
	SetupPlugin(inPSTable);
	
	// Register our event handler routine
	err = PSRegisterHandler(kEventHandlerID, (UniversalProcPtr) HandleEventForPluginModule);
	
	if (err == noErr)
	{
		// Resister our termination handler routine
		err = PSRegisterHandler(kTerminationHandlerID, (UniversalProcPtr) TerminatePluginModule);

		if (err == noErr)
		{
			err = PSSetupNewPanel(kImportantInfoPanelRsrcID, &gImportantInfoPanel);
			
			if (err == noErr)
			{
				err = SetupInfoPanel( LoWord(inRefCon) );
				if (err == noErr)
					err = PSShowPanel(gImportantInfoPanel);
			}
		}
	}

	if (err != noErr)
	{
		(void)PSErrorAlert( err, true, "\p", "\p", "\p", "\p", kQuitButtonIndex, kContinueNotQuitBtnIndex );
		(void)PSQuitShell(kDontAllowUserToContinue);
	}
	
	ExitPlugin();
}

//--------------------------------------------------------------------------------
//	TerminatePluginModule
//--------------------------------------------------------------------------------

void TerminatePluginModule ()
{	
	EnterPlugin();


	ExitPlugin();
}


//--------------------------------------------------------------------------------
//	HandleEventForPluginModule
//--------------------------------------------------------------------------------
Boolean	HandleEventForPluginModule(EventRecord *inEvent)
{
	ShellErr		err = noErr;
	Boolean			wasHandled = false;
	GrafPtr			savedPort;
	PanelItemType	itemType;
	Rect			itemRect;

	EnterPlugin();
	GetPort(&savedPort);

	switch (inEvent->what)
	{
			
		case mouseDown:
			if(FrontWindow() == gImportantInfoPanel)
			{
				SetPort(gImportantInfoPanel);
				err = HandleMouseDownInImportantInfoPanel(inEvent, &wasHandled);
			}
			else
			{
				wasHandled = PSHandleHelpWindowEvent( gHelpWindow, inEvent );
			}
			break;
			
		case osEvt:
			if ( ( (unsigned long) inEvent->message >> 24) == suspendResumeMessage )
			{
				if ( ( inEvent->message & resumeFlag ) != 0 )	// Resume event
				{ 				
					ControlHandle		printButtonControl;
					EnvironmentType		enviromentFlags;
					short				printButtonHilite = 255;
					ShellErr			err = noErr;
					GrafPtr				savedPort;

					GetPort( &savedPort );
					SetPort( gImportantInfoPanel );

					err = PSCheckEnvironment( &enviromentFlags );				// Check if a printer has become available
					if (err == noErr)
					{
						err = PSGetPanelItem( gImportantInfoPanel, kPrintButton, &itemType, (Handle *) &printButtonControl, &itemRect );
						if ( ( err == noErr ) && ( ( enviromentFlags & kPrinterAvailableMask ) == 0 ) )
							printButtonHilite = 255;
						else
							printButtonHilite = 0;
					}

					if ( FrontWindow() == gImportantInfoPanel )
						HiliteControl( ( ControlHandle ) printButtonControl, printButtonHilite );

					SetCRefCon( ( ControlHandle ) printButtonControl, printButtonHilite );		// don't remove this line!! 

					SetPort( savedPort );

					( void ) PSHandleHelpWindowEvent ( gHelpWindow, inEvent );
				}
			}
		break;
								
		default:
			break;
		}			
		

	if (err != noErr)
	{
		(void)PSErrorAlert( err, true, "\p", "\p", "\p", "\p", kQuitButtonIndex, kContinueNotQuitBtnIndex );
		(void)PSQuitShell(kDontAllowUserToContinue);
	}
		
	SetPort(savedPort);
	ExitPlugin();
	
	return wasHandled;
}
//--------------------------------------------------------------------------------
//	HandleMouseDownInImportantInfoPanel
//--------------------------------------------------------------------------------

static ShellErr	HandleMouseDownInImportantInfoPanel(EventRecord *inEvent, Boolean *wasHandled)
{
	short				itemHit, windowPart;
	ShellErr			err = noErr;
	WindowPtr			whichWindow;
	Rect				itemRect;
	Point				localPt = inEvent->where;
	HelpPrefsHandle		helpPrefs = NULL;
	PanelItemType		itemType;
	Handle				itemHandle;
	
	*wasHandled = false;

	windowPart = FindWindow(inEvent->where, &whichWindow);
	if (whichWindow == gImportantInfoPanel)
	{
		
		switch (windowPart)
		{
			case inContent:
				PSGetPanelItem(gImportantInfoPanel, kDocItem, &itemType, &itemHandle, &itemRect);
				
				GlobalToLocal(&localPt);
				if (PtInRect(localPt, &itemRect))
				{
					(void)PSHandleDocScroll(gImportantInfoPanel, localPt, kDocItem);
					*wasHandled = true;													// added to fix bug 1654999
				}
				else
				{
					if (PSGetPanelItemHit(gImportantInfoPanel, inEvent, &itemHit))
					{
						switch (itemHit)
						{
								
							case kPrintButton:
								(void)PSPrintDoc(gImportantInfoPanel, kDocItem);
								*wasHandled = true;
								break;
							
							case kSaveButton:
								PSSaveDoc(gImportantInfoPanel, kDocItem);
								*wasHandled = true;
								break;
										
							case kHelpButton:
								PSDisplayHelpWindow(gHelpWindow);
								*wasHandled = true;
								break;
										
							default:
								break;
						}
					}
				}
				break;

			default:
				break;
		}
	}
		
	return err;
}

//--------------------------------------------------------------------------------
//	SetupInfoPanel
//
//	Sets up the various user items in the panel to their correct type, specifiying
//	the data found in the ImportantInfo preference resource which is read from the ClientData
//	file.
//	This routine should be called after PSSetupNewPanel() but before PSShowPanel() for
//	the Welcome panel.
//
//	Parameters
//
//	Returns
//--------------------------------------------------------------------------------

static OSErr	SetupInfoPanel( SInt16 inPrefRsrcID )
{
	ImportantInfoPrefsHandle	prefsHandle;
	ImportantInfoPrefsPtr		prefsPtr;
	OSErr						err = noErr;
	PanelItemType				itemType;
	Rect						itemRect;

	if( inPrefRsrcID == 0 )
	{
		SInt16** theIDHandle = (SInt16**) GetResource( 'paul', 128 );	// Get Resource ID for plugin preference
		if (theIDHandle != NULL)
			inPrefRsrcID = **theIDHandle;
		else
			inPrefRsrcID = 5000;
	}

	prefsHandle = (ImportantInfoPrefsHandle)GetResource(kImportantInfoPrefsResType, inPrefRsrcID );
	
	if (prefsHandle != NULL)
	{
		if ((**prefsHandle).format == 1)
		{
			EnvironmentType		enviromentFlags;
			DocLocationType		helpDocLocation;
			Str255				helpPanelTitleStr;
			TEHandle			textHandle;
			Handle				docHandle;
			DocViewerType		docType;
			Handle				itemHandle;
			short				fontNum,
								fontStyle,
								fontSize;
								
			HLock((Handle)prefsHandle);
			prefsPtr = *prefsHandle;

			// Setup the  panel title string
			
			err = PSGetPanelItem(gImportantInfoPanel, kTitleTextItem, &itemType, &itemHandle, &itemRect);
			if (PSReadFontInfo(kFontInfoInShell, kUpgraderFonts, kLargeTextStyle, &fontNum, &fontStyle, &fontSize))
			{
				textHandle = PSNewStyledStringItem(&itemRect, prefsPtr->STRListRsrsID, 1, fontNum, fontStyle, fontSize);
				if (textHandle != NULL)
					err = PSSetPanelItem(gImportantInfoPanel, kTitleTextItem, kStyledStringType, (Handle)textHandle, &itemRect);
			}
			
			// Setup the panel main text.
			err = PSGetPanelItem(gImportantInfoPanel, kDocItem, &itemType, &itemHandle, &itemRect);
			if (err == noErr)
			{
				if ((prefsPtr->flags & kMainTextInFile) == kMainTextInFile)
					docType = kDocFileType;
				else
					docType = kDocResType;
					
				docHandle = PSNewDocViewerItem(gImportantInfoPanel, &itemRect, docType, prefsPtr->mainTextReferenceID,  prefsPtr->mainPICTBaseResID);
				if (docHandle != NULL)
					err = PSSetPanelItem(gImportantInfoPanel, kDocItem, docType, docHandle, &itemRect);
				else
					err = kCannotLoadNeededResourceErr;
			}
	
			// Check if we have printing and standard file package available, if not then
			// hide their respective versions.
			err = PSCheckEnvironment(&enviromentFlags);
			
			if (err == noErr)
			{
				if ((enviromentFlags & kPrinterAvailableMask) == 0)
				{
					ControlHandle	printButtonControl;

					err = PSGetPanelItem(gImportantInfoPanel, kPrintButton, &itemType, (Handle *)&printButtonControl, &itemRect);
					if (err == noErr)
					{
						HiliteControl(printButtonControl, 255);						
						SetCRefCon( printButtonControl, 255 );	//  don't remove this line!! 
					}
				}

				if ((enviromentFlags & kStandardFilePackageAvailableMask) == 0)
				{
					ControlHandle	saveButtonControl;

					err = PSGetPanelItem(gImportantInfoPanel, kSaveButton, &itemType, (Handle *)&saveButtonControl, &itemRect);
					if (err == noErr)
					{
						HiliteControl(saveButtonControl, 255);
						SetCRefCon( saveButtonControl, 255 );		//  don't remove this line!! 
					}
				}
			}
	
			// Read the help  contents info and setup the help window.
			GetIndString(helpPanelTitleStr, prefsPtr->STRListRsrsID, 2 );
			if ((prefsPtr->flags & kHelpTextInFile) == kHelpTextInFile)				// Check the bit mask for the location of the Help contents 
				helpDocLocation = kReadFromSimpleTextFile;							// This is of DocLocationType and should be set to one of the following
			else																	// constants.
				helpDocLocation = kReadFromResourceFile;
			
			err = PSSetupHelpWindow (helpDocLocation, prefsPtr->helpTextReferenceID, 
				prefsPtr->helpPICTBaseResID, helpPanelTitleStr, &gHelpWindow);					

			HUnlock((Handle)prefsHandle);
		}
		else
		{
			err = kUnsupportedPrefsFormatErr;
		}
	}
	else
	{
		err = kNoPrefsErr;
	}
		
	(void)PSSetPanelItemAction(gImportantInfoPanel, kContinueButton, kDefaultButtonMask);
	(void)PSSetPanelItemAction(gImportantInfoPanel, kContinueButton, kContinueButtonMask);
	(void)PSSetPanelItemAction(gImportantInfoPanel, kGoBackButton, kGoBackButtonMask);

	return err;
}
