
#include "CTextEditorWindow.h"
#include "CFileRefEditorWindow.h"
#include "CSALEditorWindow.h"
#include "EditorUtilities.h"

// Dialog panes
const ResType 	kPanelTitleET 					= 'ptsE';
const ResType 	kBackgroundPicET 				= 'bpiE';
const ResType 	kFirstHelpPicET 				= 'pihE';
const ResType 	kEditMainTextB 					= 'emtB';
const ResType 	kEditHelpTextB 					= 'ethB';
const ResType 	kEmbedHelpTextRB 				= 'emhR';
const ResType 	kFileHelpTextRB 				= 'sfhR';
const ResType 	kRemoveB 						= 'rmvB';
const ResType 	kLeaveAppRunningCB 				= 'larC';
const ResType 	kEditAppFileRefB 				= 'eafB';
const ResType 	kEditDocFileRefB 				= 'edfB';
const ResType 	kShowLaunchCheckboxCB 			= 'slcC';
const ResType 	kPreSelectLaunchCheckboxCB 		= 'lcdC';
const ResType 	kLaunchCheckboxTitleET 			= 'lctE';


// ---------------------------------------------------------------------------
//		 CEditorWindow
// ---------------------------------------------------------------------------
CEditorWindow::CEditorWindow( SInt16 inFileRefNum, SInt16 inPreferenceRsrcID, SInt16 inResListRsrcID )
	: StDialogHandler(200, NULL), mFileRefNum(inFileRefNum), mPreferenceRsrcID(inPreferenceRsrcID), mResListRsrcID(inResListRsrcID), mFlags(0), mStringListRsrcID(0), mMainTextRsrcID(0), mHelpTextRsrcID(0), mAppFileRefRsrcID(0), mDocFileRefRsrcID(0)

{

	if( mPreferenceRsrcID == 0 ) {
		SInt16** theIDHandle = (SInt16**) GetIndResource( 'paul', 1 );	//Get Resource ID for plugin preference
		if (theIDHandle != NULL)
			mPreferenceRsrcID = **theIDHandle;
		else
			mPreferenceRsrcID = 3500;
	}

}

// ---------------------------------------------------------------------------
//		 ~CEditorWindow
// ---------------------------------------------------------------------------
CEditorWindow::~CEditorWindow()
{
}


// ---------------------------------------------------------------------------
//		 DoEdit
//
//		Opens the editor window and handles all editing actions.  Returns
//		a value indicating how the user finished the editing task
//
// ---------------------------------------------------------------------------
SInt32	CEditorWindow::DoEdit()
{
	
	SInt32			theResult = -1;
	LWindow*		theDialog = GetDialog();

	this->LoadWindowFromFile();
	
	theDialog->Show();
	
	// Set active field
	LEditField*	theField = (LEditField*) mDialog->FindPaneByID(kPanelTitleET);
	SignalIf_(theField == nil);
	mDialog->SetLatentSub(theField);

	
	while (true) {
		MessageT	hitMessage = DoDialog();
		
		if (hitMessage == msg_Cancel) {									// Cancel button
			theResult = 1;
			break;
			
		} else if (hitMessage == msg_OK) {								// OK button
			this->SaveWindowToFile();
			theResult = 0;
			break;
			
		} else if (hitMessage == kRemoveB) {							// Remove button
			theResult = 2;
			break;
		
		} else if (hitMessage == kEditMainTextB) {						// Edit main text
				DoEditText( mFileRefNum, mMainTextRsrcID  );
				
		} else if (hitMessage == kEditHelpTextB) {						// Edit help text
		
			if( mDialog->FindPaneByID(kEmbedHelpTextRB)->GetValue() )
				DoEditText( mFileRefNum, mHelpTextRsrcID  );
			else
				DoEditFileRef( mFileRefNum, mHelpTextRsrcID );
				
		} else if (hitMessage == kEditAppFileRefB) {					// Edit application location
			DoEditFileRef( mFileRefNum, mAppFileRefRsrcID );
		
		} else if (hitMessage == kEditDocFileRefB) {					// Edit document location
			DoEditFileRef( mFileRefNum, mDocFileRefRsrcID );
				
		} else {														// Idle time tasks
		
			if( mDialog->FindPaneByID(kEmbedHelpTextRB)->GetValue() )
				mDialog->FindPaneByID(kFirstHelpPicET)->Enable();
			else
				mDialog->FindPaneByID(kFirstHelpPicET)->Disable();
		
			if( mDialog->FindPaneByID(kShowLaunchCheckboxCB)->GetValue() ) {
				mDialog->FindPaneByID(kPreSelectLaunchCheckboxCB)->Enable();
				mDialog->FindPaneByID(kLaunchCheckboxTitleET)->Enable();
			} else {
				mDialog->FindPaneByID(kPreSelectLaunchCheckboxCB)->Disable();
				mDialog->FindPaneByID(kLaunchCheckboxTitleET)->Disable();
			}

		}
	}
	
	return theResult;
}


// ---------------------------------------------------------------------------
//		 SaveWindowToFile
//
//		Writes the contents of the window out to the plug-in's preference
//		resource.
//
// ---------------------------------------------------------------------------
void CEditorWindow::SaveWindowToFile()
{
	SInt16	savedResFile = CurResFile();
	UseResFile( mFileRefNum );
	
	TRsrcHandle* theRsrcStream = new TRsrcHandle();
	SignalIf_(theRsrcStream == nil);
	
	if( mStringListRsrcID == 0 )
		 mStringListRsrcID = GetUniqueIDForResType( mFileRefNum, 'STR#' );		 

	theRsrcStream->AppendShort( 3 );	// Append Format

	// Main Text flag
	/*
	LStdRadioButton* theRadioButton = (LStdRadioButton*) mDialog->FindPaneByID(kEmbedMainTextRB);
	SignalIf_(theRadioButton == nil);
	if( theRadioButton->GetValue() )
		mFlags &= ~kMainTextInFile;
	else
		mFlags |= kMainTextInFile;
*/
	// Help Text flag
	LStdRadioButton* theRadioButton = (LStdRadioButton*) mDialog->FindPaneByID(kEmbedHelpTextRB);
	SignalIf_(theRadioButton == nil);
	if( theRadioButton->GetValue() )
		mFlags &= ~kHelpTextInFile;
	else
		mFlags |= kHelpTextInFile;


	// Leave app running flag
	LStdCheckBox*	theCheckbox = (LStdCheckBox*) mDialog->FindPaneByID(kLeaveAppRunningCB);
	SignalIf_(theCheckbox == nil);
	if( theCheckbox->GetValue() )
		mFlags |= kKeepApplicationOpen;
	else
		mFlags &= ~kKeepApplicationOpen;

	// Show Launch Checkbox
	theCheckbox = (LStdCheckBox*) mDialog->FindPaneByID(kShowLaunchCheckboxCB);
	SignalIf_(theCheckbox == nil);
	if( theCheckbox->GetValue() )
		mFlags |= kShowLaunchCheckbox;
	else
		mFlags &= ~kShowLaunchCheckbox;

	// Defalut checkbox state
	theCheckbox = (LStdCheckBox*) mDialog->FindPaneByID(kPreSelectLaunchCheckboxCB);
	SignalIf_(theCheckbox == nil);
	if( theCheckbox->GetValue() )
		mFlags |= kPrecheckLaunchCheckbox;
	else
		mFlags &= ~kPrecheckLaunchCheckbox;


	theRsrcStream->AppendShort( mFlags );
	theRsrcStream->AppendShort( mMainTextRsrcID );
	theRsrcStream->AppendShort( mHelpTextRsrcID );


	// First Help pict value
	LEditField* theField = (LEditField*) mDialog->FindPaneByID(kFirstHelpPicET);
	SignalIf_(theField == nil);
	SInt16	theHelpFirstPictRsrcID = theField->GetValue();
	theRsrcStream->AppendShort( theHelpFirstPictRsrcID );
	
	// Background pict value
	theField = (LEditField*) mDialog->FindPaneByID(kBackgroundPicET);
	SignalIf_(theField == nil);
	SInt16	 theBackgroundPictRsrcID = theField->GetValue();
	theRsrcStream->AppendShort( theBackgroundPictRsrcID );
	
	// Panel name
	theField = (LEditField*) mDialog->FindPaneByID(kPanelTitleET);
	SignalIf_(theField == nil);
	Str255	theStr;
	theField->GetDescriptor( theStr );
	WriteStringListResourceIndex( mFileRefNum, theStr, mStringListRsrcID, 1 );
	theRsrcStream->AppendShort( mStringListRsrcID );
			
	// Help window title string
	UseResFile( savedResFile );
	GetIndString( theStr, 233, 2 );
	UseResFile( mFileRefNum );
	
	WriteStringListResourceIndex( mFileRefNum, theStr, mStringListRsrcID, 2 );			
	
	// Launch Checkbox name
	theField = (LEditField*) mDialog->FindPaneByID(kLaunchCheckboxTitleET);
	SignalIf_(theField == nil);
	theField->GetDescriptor( theStr );
	WriteStringListResourceIndex( mFileRefNum, theStr, mStringListRsrcID, 3 );
	
			
	// App File Ref
	theRsrcStream->AppendShort( mAppFileRefRsrcID );
	
	// Doc File Ref
	theRsrcStream->AppendShort( mDocFileRefRsrcID );

	
	// Build Res list resource
	ResetResListResource( mFileRefNum, mResListRsrcID );
	AppendToResListResource( mFileRefNum, mResListRsrcID, 'slpr', mPreferenceRsrcID );
	AppendToResListResource( mFileRefNum, mResListRsrcID, 'STR#', mStringListRsrcID );
	
	if( mFlags & kHelpTextInFile ) {
		AppendToResListResource( mFileRefNum, mResListRsrcID, 'flrf', mHelpTextRsrcID );
	} else {
		AppendToResListResource( mFileRefNum, mResListRsrcID, 'TEXT', mHelpTextRsrcID );
		AppendToResListResource( mFileRefNum, mResListRsrcID, 'styl', mHelpTextRsrcID );
		AppendToResListResource( mFileRefNum, mResListRsrcID, 'PICT', theHelpFirstPictRsrcID );
	}
	
	if( mFlags & kMainTextInFile ) {
		AppendToResListResource( mFileRefNum, mResListRsrcID, 'flrf', mMainTextRsrcID );
	} else {
		AppendToResListResource( mFileRefNum, mResListRsrcID, 'TEXT', mMainTextRsrcID );
		AppendToResListResource( mFileRefNum, mResListRsrcID, 'styl', mMainTextRsrcID );
	}
	AppendToResListResource( mFileRefNum, mResListRsrcID, 'PICT', theBackgroundPictRsrcID );
	AppendToResListResource( mFileRefNum, mResListRsrcID, 'PICT', theBackgroundPictRsrcID  + 1 );

	AppendToResListResource( mFileRefNum, mResListRsrcID, 'flrf', mAppFileRefRsrcID );
	AppendToResListResource( mFileRefNum, mResListRsrcID, 'flrf', mDocFileRefRsrcID );

	theRsrcStream->Write( mFileRefNum, 'slpr', mPreferenceRsrcID );

			
	delete theRsrcStream;
	
	UseResFile( savedResFile );
}


// ---------------------------------------------------------------------------
//		 LoadWindowFromFile
//
//		Reads plug-in's preference resource and fills in fields of editor window.
//		If the resource hasn't been created yet, then routine provides default
//		values for the fields.
//
// ---------------------------------------------------------------------------
void CEditorWindow::LoadWindowFromFile()
{
	
	SInt16	savedResFile = CurResFile();
	UseResFile( mFileRefNum );
	
	Handle 		theRsrcHandle = Get1Resource( 'slpr', mPreferenceRsrcID );
	
	if( theRsrcHandle )
	{
		TRsrcHandle* theRsrcStream = new TRsrcHandle( theRsrcHandle );
		SignalIf_(theRsrcStream == nil);
	
		if( theRsrcStream->ReadShort() == 3)
		{
		
			mFlags 							= theRsrcStream->ReadShort();

			// Set Embedded Main Text Radio Button
			/*
			LStdRadioButton* theRadioButton = (LStdRadioButton*) mDialog->FindPaneByID(kEmbedMainTextRB);
			SignalIf_(theRadioButton == nil);
			theRadioButton->SetValue( (mFlags & kMainTextInFile) != kMainTextInFile );

			// Set File Help Main Radio Button
			theRadioButton = (LStdRadioButton*) mDialog->FindPaneByID(kFileMainTextRB);
			SignalIf_(theRadioButton == nil);
			theRadioButton->SetValue( (mFlags & kMainTextInFile) == kMainTextInFile );

*/
			// Set Embedded Help Text Radio Button
			LStdRadioButton* theRadioButton = (LStdRadioButton*) mDialog->FindPaneByID(kEmbedHelpTextRB);
			SignalIf_(theRadioButton == nil);
			theRadioButton->SetValue( (mFlags & kHelpTextInFile) != kHelpTextInFile );

			// Set File Help Text Radio Button
			theRadioButton = (LStdRadioButton*) mDialog->FindPaneByID(kFileHelpTextRB);
			SignalIf_(theRadioButton == nil);
			theRadioButton->SetValue( (mFlags & kHelpTextInFile) == kHelpTextInFile );

			mMainTextRsrcID					= theRsrcStream->ReadShort();
			mHelpTextRsrcID					= theRsrcStream->ReadShort();

			// Set First Help pict value
			LEditField* theField = (LEditField*) mDialog->FindPaneByID(kFirstHelpPicET);
			SignalIf_(theField == nil);
			theField->SetValue( theRsrcStream->ReadShort() );
			
			// Set Background pict value
			theField = (LEditField*) mDialog->FindPaneByID(kBackgroundPicET);
			SignalIf_(theField == nil);
			theField->SetValue( theRsrcStream->ReadShort() );
			
			// Set Panel name
			theField = (LEditField*) mDialog->FindPaneByID(kPanelTitleET);
			SignalIf_(theField == nil);
			mStringListRsrcID = theRsrcStream->ReadShort(); 
			Str255	thePanelTitleStr;
			GetIndString( thePanelTitleStr, mStringListRsrcID, 1 );
			theField->SetDescriptor( thePanelTitleStr );

			// Set Launch Checkbox title
			theField = (LEditField*) mDialog->FindPaneByID(kLaunchCheckboxTitleET);
			SignalIf_(theField == nil);
			Str255	theLaunchCheckboxTitleStr;
			GetIndString( theLaunchCheckboxTitleStr, mStringListRsrcID, 3 );
			theField->SetDescriptor( theLaunchCheckboxTitleStr );


			// Leave app running flag
			LStdCheckBox*	theCheckbox = (LStdCheckBox*) mDialog->FindPaneByID(kLeaveAppRunningCB);
			SignalIf_(theCheckbox == nil);
			theCheckbox->SetValue( (mFlags & kKeepApplicationOpen) == kKeepApplicationOpen );

			// Show Launch Checkbox
			theCheckbox = (LStdCheckBox*) mDialog->FindPaneByID(kShowLaunchCheckboxCB);
			SignalIf_(theCheckbox == nil);
			theCheckbox->SetValue( (mFlags & kShowLaunchCheckbox) == kShowLaunchCheckbox );

			// Default Launch Checkbox state
			theCheckbox = (LStdCheckBox*) mDialog->FindPaneByID(kPreSelectLaunchCheckboxCB);
			SignalIf_(theCheckbox == nil);
			theCheckbox->SetValue( (mFlags & kPrecheckLaunchCheckbox) == kPrecheckLaunchCheckbox );

			// App File Ref
			mAppFileRefRsrcID = theRsrcStream->ReadShort();
			
			// Doc File Ref
			mDocFileRefRsrcID = theRsrcStream->ReadShort();

		}
		
		delete theRsrcStream;

	} else {
		mFlags						= 0;
		mStringListRsrcID			= 0;
		mMainTextRsrcID				= 0;
		mHelpTextRsrcID				= 0;
		mAppFileRefRsrcID			= 0;
		mDocFileRefRsrcID			= 0;
	}
	
	UseResFile( savedResFile );
}



