/*
	File:		UpgraderPlugin.h

	Contains:	All common function defintions and structures
				between the upgrader and the plugins

	Version:	Upgrader 1.1

	Written by:	Humphrey Looney

	Copyright:	 1996-1997 by Apple Computer, Inc., all rights reserved.

	File Ownership:

		DRI:				Kevin Aitken

		Other Contact:		Mark Minshull

		Technology:			Installer

	Writers:

		(KBA)	Kevin Aitken
		(HL)	Humphrey Looney
		(CJB)	Canice Bowe.

	Change History (most recent first):

		<38>	 8/21/97	KBA		Added support for new plug-in routine, PSForceRestart().
		<37>	 7/12/97	KBA		Changed return type of PSNewDocViewerItem() and added
									PSMakeFSSpecFromFileRefID().
	<34>	01-May-97	HL	Tidied up error numbers. 
	<33>	17-Apr-97	CJB	Changed the base offset for the 'finf' items from 1 to 0.
	<32>	17-Apr-97	CJB	Added in a new 'finf' style constant for Alert text items.
	<31>	16-Apr-97	CJB	Removed obsolete prototypes : PSGetPanelItemRect, PSGetPanelItemData, PSGetPanelItemControl,
							and PSSetPanelUserItem.
							Added in/changed  prototypes : PSGetPanelItem, PSSetPanelItem, PSNewStyledStringItem,
							PSNewStyledTextItem, PSNewUserItem and PSNewDocViewerItem.	
						HL	Added PSActivate function
	<30>	04-Apr-97	CJB Removed obsolete prototypes and definitions, PSGrowPanel(), PSGrowPanelItems()
							kLanguageSelectType, kHasScrollBar, kIsDialog.
							Added constant kHelpPanel which can be used in the same places as kGlobalPanel.
							Merged contents of PluginStubs.h as this file will be obsoleted.
	<29>	07-mar-97	sh	- change PSReadFontInfo() prototype to return seperate font face, style and size, rather than a fontInfoRecPtr
	<28>	06-mar-97	sh	- removed stuff for PSOpenHelpWindow() and added-in stuff for PSSetupHelpWindow() and PSDisplayHelpWindow()
						hl  - added new error message for kNextPluginSameAsCurrentErr also DocumentType enum for
							  PSAlert also kUserContinuingAfterRestartAlertMsgNum 				    
	<27>	5-Mar-97	HL	Added kStopAndQuitDLOGID definition
	<26>	  3/2/97	KBA		Modified conclusion definitions to support changes in conclusion
									plugin.  Added foreced restart global data type.
	<25>	27-Feb-97	CJB	Added in contants kReadFromResourceFile and kReadFromSimpleText file.
	<24>	27-Feb-97	CJB	Changed refCon type for PSGetPRefCon and PSSetPRefCon from UInt32 to long so that
							it matches the WindowManager refCon field.
	<23>	21-Feb-97	CJB	Added in a 'transparent' parameter to PSSetPanelUserItem() call.
	<22>	18-feb-97	sh	- added in stuff for PSOpenHelpWindow(), PSCloseHelpWindow(), PSHandleHelpWindowEvent() and PSCheckEnvironment()
	<21>	07-feb-97	sh	- added in PSReadFontInfo() stuff
	<20>	07-Feb-97	CJB	Changed PSLaunchFile to return a Boolean instead of an error as it handles and 
							displays its own error messsages.
	<19>	06-Feb-97	CJB	Added in PSCollect prototype.
	<18>	04-Feb-97	CJB	Changed prototype for all DocViewer related call so that they accept a
							panel item number.
	<17>	 1/23/97	KBA	Added parameters to PSLaunchFile() and defines for
							kLaunchAppInFront and kLaunchAppInBack to be used with
							PSLaunchFile().
	<16> 	22-Jan-97	HL	Added an error number kUnknownPluginNameErr
	<15>	21-Jan-97	CJB	Added prototype for PSUpdatePanel.
							Added in global data types for selected volume vRefNum and conclusion
							plugin settings.
	<14>	 1/10/97	KBA	Removed defines for dialog IDs 509, 517, 521, and 522 and their
							button definitions because they are either obsolete or owned by
							the installation plugin.
	<10>	17-Dec-96	CJB	Made panel item types and their definitions public so that they can be
							used in the PSSetPanelItem() call.
	<09>	10-Dec-96	CJB	Removed PSUpdatePanel prototype as it is no longer in the API.
	<08>	19-Nov-96	HL  Merged contents of ErrorIDMap.h and UpgraderErrorMgt.h to this file
							also included REZ preprocessor lines, removed some of the message codes 
							from Bonjour as these will be contained in the installer plugin changed
							definition of kStopDLOGID to kOKDLOGID to make this dialog a more general
							purpose alert box. Added definition of kPluginError to pass to PSErrorAlert.
	<07>	16-Nov-96	CJB	Added in new panel creation flag, 'kHasGoAwayBox' as this varies depending on
							the panel.
	<06>	14-Nov-96	CJB	Changed prototype of PSGetPanelItemRect and PSGetPanelItemData so they no longer
							rely on the PanelItemList definition which is internal to the Shell.
							Moved PanelItemList definition to Panels.h which is internal to the Upgrader shell.
							Removed PSGetPanelItem and GetPanelWindow from the API as it is an internal function.
	<05>	13-Nov-96	CJB	Added in kHelpButtonMask and kDefaultButtonMask
	<04>	09-Nov-96	CJB	Added in 'flags' field to PanelItemList struct and some
							bit masks for it.
	<03>	08-Nov-96	HL	Changed prototype for PSHidePanel
	<02>	08-Nov-96	CJB	Added in prototypes for PSGetPRefCon and PSSetPRefCon. Removed obsolete prototypes
							for PSChangePanel and PSPlugInRequestedQuit.
	<01>	07-Nov-96	HL  Replacing extern before all function definitions
							this was causing trouble with c++ modules also
							added #ifdef __cplusplus
	<00>	06-Nov-96	HL	New
			Copied all this from the obsoleted InstallerPlugin.h file 
			which is now called PluginStub.h

	To Do:
*/

#ifndef __UPGRADERPLUGIN__
#define __UPGRADERPLUGIN__

#ifndef REZ
#include <AppleEvents.h>
#include <Dialogs.h>
#endif

#if defined(__MWERKS__)		// <30>
#include <SetUpA4.h>
#include <A4Stuff.h>
#endif

// General Shell error numbers that may be returned to plugins
#define kCouldNotFindPluginMsgNum					1000
#define kUnknownPlugInHandlerErr					1001				
#define kNoHighLevelEventsErr						1002				
#define	kCouldNotFindResourceMsgNum					1003
#define kMemoryErrorMsgNum							1004	
#define	kCantSaveDocumentErr						1005		// PSSaveDoc cannot save the specified DocViewer object.
#define	kForceRestartWarningMsgNum					1006
#define	kAppNotFoundMsgNum							1007
#define	kDocNotFoundMsgNum							1008
#define	kAppCouldNotBeLaunchUnknownProblemMsgNum	1009
#define	kAppCouldNotBeLaunchNoEnoughMemoryMsgNum	1010
#define	kAppCouldNotBeLaunchFileOpenMsgNum			1011
#define	kRightDiskNameWrongDiskInsertedMsgNum		1012
#define	kWrongDiskInsertedMsgNum					1013
#define	kFileNotFoundOnServerMsgNum					1014
#define kCantEjectCDDiskMsgNum						1015

// Errors Returned by PSQuitShell
#define kUserContinuingAfterRestartAlertMsgNum		1030

// Plugin sequence errors 
#define kUnknownPluginNameErr						1040		// <16> 
#define kNextPluginSameAsCurrentErr					1041		// <28>

// Global data manager errors
#define kUnknownGlobalDataErr						2000
#define kGlobalDataOutOfMemErr						2001

#define	kUnsupportedPrefsFormatErr					2010
#define	kNoPrefsErr									2011

// Panel manager errors												
#define kPanelItemNotFoundInListErr					2050				
#define kCannotLoadNeededResourceErr				2051
#define kNoDataAvailableForItemErr					2052
#define kInternalErr								2053
#define kNoGlobalPanelErr							2054
#define kItemTypeMismatchErr						2055
#define kCantFindTextRsrcErr						2056
#define kItemNotLocalisedErr						2057
#define	kCantChangePanelItemToSpecifiedTypeErr		2058

// Dialog IDs : these are needed by plugins to define there own private 'ners' resources
#define	kStopOrContinueDLOGID						506	// note, center main
#define	kQuitOrContinueDLOGID						507	// note, center main
#define	kNoteOKDLOGID								508	// note, center main
#define	kRestartOrContinueDLOGID					514	// note, center main
#define	kInsertDiskDLOGID							515	// disk swap icon, center main
#define	kStopOKDLOGID								519	// stop, center main
#define	kSkipOrContinueDLOGID						520	// caution, alert parent
#define kStopDLOGID									600 // OK, center main *note kStopOKDLOGID
														// shows a continue button as well as OK 
#define kStopAndQuitDLOGID							526 // <27>
#ifndef REZ
#ifdef __cplusplus
extern "C" {
#endif

/* PSErrorAlert related constants */
#define kEmptyString (ConstStr255Param)"\p"

// Constants that are used in PSErrorAlert to define the default button and the selected button
#define kOKButtonIndex							1
// Skip or continue
#define	kContinueNotSkipBtnIndex   				1
#define	kSkipNotContinueBtnIndex   				2
// Continue or Quit
#define	kQuitButtonIndex						1
#define	kContinueNotQuitBtnIndex	 			2
// Continue or Restart
#define	kRestartButtonIndex			 			1
#define	kContinueNotRestartBtnIndex	 			2
// yes or no
#define	kYesButtonIndex							1
#define	kNoButtonIndex							2

// constants to indicate to PSErrorAlert whether to look in the
// plugins resource file for the error or in the Shell resource file
#define kStandardShellError	true			// Tells PSErrorAlert to look in Shell for error
#define kPluginError		false			// Tells PSErrorAlert to look in Plugin res fork for error

// constants for PSQuitShell
#define kDontAllowUserToContinue	false
#define kAllowUserToContinue		true

//------------------------------------------------------------------------------------------------------
//	G L O B A L   D A T A   D E F I N I T I O N S <15>
//------------------------------------------------------------------------------------------------------
typedef	OSType			GlobalDataType;
typedef	Ptr				GlobalDataPtr;

enum										// This is the list of global data types shared between plugin's
											// NOTE. plugin's can store their own global data as well.
	{
	kTargetDiskVolRefNumDataType =	'trgt',	// The selected volume vRefNum as set by Target Selectin plugin (SInt16).
	kForceRestartOnQuitDataType = 	'rsrq'	// Global data identifier to singal to shell and plugins that a restart is required (Boolean).
	};


//------------------------------------------------------------------------------------------------------
//	P L U G I N   D E F I N I T I O N S
//------------------------------------------------------------------------------------------------------
#define	kEventHandlerID			'ehID'
#define	kTerminationHandlerID	'thID'

typedef OSErr 			ShellErr;
typedef	OSType			HandlerIDType;
typedef SInt16 			ResourceID;

enum	
	{
	kUseDefaultNextModuleName = 0
	};

//------------------------------------------------------------------------------------------------------
//	P A N E L  M A N A G E R  D E F I N I T I O N S
//------------------------------------------------------------------------------------------------------
typedef WindowPtr 		PanelPtr;

#define	kGlobalPanel	(void *)0xFFFFFFFF	// Pass this constant to PSShowPanel/PSHidePanel if the current
											// global panel is unknown (eg. when the plugin does not itself display
											// a panel).
#define	kHelpPanel  	(void *)0xFFFFFFFE
	
enum										// Definitions for the 'inFlags' parameter to PSNewCustomPanel()
	{
	kGrowWindow 	= 0x02,
	kIsGlobal 		= 0x04,
	kHasGoAwayBox	= 0x10					// <07>
	};	

typedef	UInt8			DVFlags;

//  Type and  definitions for the panel item types
typedef unsigned long		PanelItemType;
typedef PanelItemType		DocViewerType;		// <31> subset of PanelItemType
enum
	{
	kStyledTextType		= 'STXT',				// 'TEXT' and 'styl' pairs
	kStyledStringType	= 'SSTR',				// 'STR#', index and 'finf'
	kPICTType			= 'PICT',
	kDocResType			= 'DOCV',				// 'TEXT', 'styl' and 'PICT's stored in resources
	kDocFileType		= 'DOCF',				// SimpleText file will optional 'styl' and 'PICT' in resource fork.
	kControlType		= 'CNTL',
	kIconType			= 'ICON',
	kUserItemType		= 'USER'				// User panel item for drawing custom items.
	};

enum 											// <28> Used in PSAlert
{
	kUpgraderFile = 0,
	kClientDataFile,
	kCurrentPluginResFile
};

typedef short DocumentType;						// Used in PSAlert

typedef Byte				DocLocationType;	// kReadFromResourceFile, kReadFromSimpleTextFile					// <22>
enum {																												// <07>
	kPrinterAvailableMask = 0x00000001,
	kStandardFilePackageAvailableMask = 0x00000002
};

typedef	unsigned long		EnvironmentType;																		// <22>
enum {																												// <22>
	kReadFromResourceFile = true,				// <25>
	kReadFromSimpleTextFile = false
};


//  Type and bit definitions for the action panel item attribute
typedef	UInt16			PanelActionType;

enum
	{
	kContinueButtonMask = 0x0001,
	kGoBackButtonMask 	= 0x0002,
	kQuitButtonMask		= 0x0004,
	kHelpButtonMask		= 0x0008,
	kDefaultButtonMask	= 0x0010
	};

typedef Handle PanelUserItemHandle;				// <31>


// Build in 'finf' resource ID's for the fontInfo field of StyledStringDesc.
enum
	{
	kUpgraderFonts	= 128,
	kLargeTextStyle = 0,						// <33>
	kMediumTextStyle,
	kSmallTextStyle,
	kAlertTextStyle								// <32>
	};
	
enum {		// where the 'finf' is to be read from
	kFontInfoInClientDataFile,
	kFontInfoInPluginFile,
	kFontInfoInShell,
	kFontInfoInAnyFile
};

// Constants to use with PSLaunchFile
#define kLaunchAppInFront	true				// Tells PSLaunchFile to bring the launched application to the front
#define kLaunchAppInBack	false				// Tells PSLaunchFile to leave the launched application in the back.

struct FontInfoRec {							// <21>
	short		font;
	short		style;
	short		size;
};
typedef struct FontInfoRec FontInfoRec, *FontInfoRecPtr;

/* Macros to customize Metrowerks code resource macros */
// Call this Macro whenever control is swithched to the plugin
#if defined(__MWERKS__)									// <30>
#define	EnterPlugin()							\
		EnterCodeResource();
#endif

// Call this Macro whenever control is from the plugin to the shell
#if defined(__MWERKS__)									// <30>
#define ExitPlugin()							\
		ExitCodeResource()
#endif
	
extern 	QDGlobals 				*gQDPtr;				// <30>


/* PROTOTYPES */

// Global Data Manager calls
extern ShellErr 		PSSetGlobalData( GlobalDataType inGlobalDataType, GlobalDataPtr inGlobalDataPtr, Size inDataSize );
extern ShellErr 		PSGetGlobalData( GlobalDataType inGlobalDataType, GlobalDataPtr inGlobalDataPtr, 
										Size inMaxDataSize, Size *outActualDataSize );

// Plugin management calls
extern ShellErr 		PSGoToPreviousPlugin( void );
extern ShellErr 		PSGoToNextPlugin( ResourceID inStrResID );
extern ShellErr			PSQuitShell( Boolean canAllowUserToContinue );		// <03>
extern ShellErr			PSForceRestart();									// <38>
extern ShellErr			PSRegisterHandler( HandlerIDType inHandlerType, UniversalProcPtr inHandlerProcPtr );

// Panel management calls
extern ShellErr			PSSetupNewPanel( SInt16 inPanelItemsRsrcID, PanelPtr *outPanelPanelPtr );
extern ShellErr			PSNewCustomPanel( short inPanelItemsRsrcID, DVFlags inFlags, PanelPtr *outPanelPanelPtr );
extern ShellErr			PSUpdatePanel( PanelPtr inPanel );
extern void				PSDisposePanel( PanelPtr inThePanel );
extern ShellErr 		PSShowPanel(PanelPtr inPanel );
extern void				PSHidePanel( PanelPtr inPanel );
extern Boolean			PSGetPanelItemHit( PanelPtr inPanel, EventRecord *inPanelEvent, short *outItemHit);
extern ShellErr			PSSetPanelItem(PanelPtr inPanel, short inItemNumber, PanelItemType inItemType, Handle inItemHandle, Rect *inItemRect);
extern ShellErr 		PSGetPanelItem(PanelPtr inPanel, short inItemNumber, PanelItemType *outItemType, Handle *outItemHandle, Rect *outItemRect);
extern void				PSSetPRefCon(PanelPtr inPanel, long inRefCon);
extern long				PSGetPRefCon(PanelPtr inPanel);
extern ShellErr			PSSetPanelItemAction(PanelPtr panel, short itemNumber, PanelActionType actionType);

extern TEHandle 		PSNewStyledStringItem(Rect *inItemRect, short inSTRListRsrcID, short inStringListItem, short inFontNum, short inFontStyle, short inFontSize);
extern TEHandle			PSNewStyledTextItem(Rect *inItemRect, short inTEXTRsrcID);
extern PanelUserItemHandle PSNewUserItem(UserItemProcPtr inUserItem, Boolean transparent);
extern Handle			PSNewDocViewerItem(PanelPtr inPanel, Rect *inItemRect, DocViewerType inDocViewerType, short inTextRsrcID, short inBasePICTRsrcID);
extern ShellErr			PSActivatePanel(PanelPtr inPanel, Boolean shouldActivate);  
              		
// DocViewer item related calls
extern ShellErr			PSHandleDocScroll(PanelPtr inPanel, Point localPt, short inDocItem);
extern void				PSSaveDoc( PanelPtr inPanel, short inDocItem) ;
extern ShellErr			PSPrintDoc( PanelPtr inPanel, short inDocItem );

// Standard help windows calls
extern OSErr			PSSetupHelpWindow ( DocLocationType location, short resID, short basePICTResID, Str255 helpPanelTitleStr, PanelPtr *outHelpPanel );		// <28>
extern void				PSDisplayHelpWindow ( PanelPtr inHelpPanel );																							// <28>
extern void				PSCloseHelpWindow ( PanelPtr helpPanel );																	// <22>
extern Boolean			PSHandleHelpWindowEvent ( PanelPtr inHelpPanel, EventRecord *inEvent );										// <22>

// Utility calls
extern OSErr			PSCheckEnvironment ( EnvironmentType *theEnvironment );														// <22>
extern Boolean			PSReadFontInfo ( short location, short inResID, short inFontItem, short *outFontFace, short *outFontStyle, short *outFontSize );	// <21>		// <29>
extern Boolean			PSLaunchFile(SInt16 inAppFileRefRsrcID, SInt16 inDocFileRefRsrcID, AEDescList *inOptionalOpenParams, Boolean inLaunchAppInFront, ProcessSerialNumber *outApplicationPSN);
extern Boolean			PSMakeFSSpecFromFileRefID(SInt16 inFileRefID, Boolean inShowErrorAlert, FSSpec *outFoundFile );
extern SInt16 			PSErrorAlert( SInt16 errNum, Boolean isStandardShellErr, ConstStr255Param p0, ConstStr255Param p1, ConstStr255Param p2, ConstStr255Param p3, SInt16 inDefaultButton, SInt16 inCancelButton );
extern void 			PSCollect(SInt16 inResListRsrcID);											// <19>
extern SInt16 			PSAlert(short DLOGID, DocumentType whichFileContainsDLOG, ConstStr255Param alertText, ConstStr255Param 	p0, ConstStr255Param p1, ConstStr255Param p2, ConstStr255Param p3, SInt16 inDefaultButton, SInt16 inCancelButton );

	
/* Prototypes for the 3 required plug-in handler routines */
void		InitializePluginModule		(	void* inPSTable, 
											SInt32 inRefCon, 
											Boolean inEnterAtBeginning );
Boolean		HandleEventForPluginModule	(	EventRecord *inEvent );
void 		TerminatePluginModule		(	void );

/* Macro so plugin main entry point conforms to ERS */
#define	 InitializePluginModule main					// <30>	

void	SetupPlugin( void *shellFunctions );

#ifdef __cplusplus
}
#endif
#endif					//REZ												
#endif