/*
 	File:		TextEncodingPlugin.h
 
 	Contains:	Required interface for Text Encoding Converter-Plugins
 
 	Version:	
 
 	DRI:		Tom Naughton
 
 	Copyright:	 1984-1996 by Apple Computer, Inc.
 				All rights reserved.
 
 	Warning:	*** APPLE INTERNAL USE ONLY ***
 				This file may contain unreleased API's
 
 	BuildInfo:	Built by:			Sue Kuo
 				With Interfacer:	2.0d13   (PowerPC native)
 				From:				TextEncodingPlugin.i
 					Revision:		18
 					Dated:			4/13/97
 					Last change by:	TRN
 					Last comment:	1647169: removed localized name API modified
 
 	Bugs:		Report bugs to Radar component System Interfaces, Latest
 				List the version information (from above) in the Problem Description.
 
*/
#ifndef __TEXTENCODINGPLUGIN__
#define __TEXTENCODINGPLUGIN__

#ifndef __TEXTENCODINGCONVERTER__
#include <TextEncodingConverter.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_IMPORT_SUPPORTED
#pragma import on
#endif

#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif

/*
####################################################################################
		Structs
####################################################################################
*/
struct TECBufferContextRec {
	TextPtr 						textInputBuffer;
	TextPtr 						textInputBufferEnd;
	TextPtr 						textOutputBuffer;
	TextPtr 						textOutputBufferEnd;

	TextEncodingRunPtr 				encodingInputBuffer;
	TextEncodingRunPtr 				encodingInputBufferEnd;
	TextEncodingRunPtr 				encodingOutputBuffer;
	TextEncodingRunPtr 				encodingOutputBufferEnd;

};
typedef struct TECBufferContextRec TECBufferContextRec;

struct TECPluginStateRec {

	UInt8 							state1;
	UInt8 							state2;
	UInt8 							state3;
	UInt8 							state4;

	UInt32 							longState1;
	UInt32 							longState2;
	UInt32 							longState3;
	UInt32 							longState4;

};
typedef struct TECPluginStateRec TECPluginStateRec;

struct TECConverterContextRec {

																/* public - manipulated externally and by plugin*/
	Ptr 							pluginRec;
	TextEncoding 					sourceEncoding;
	TextEncoding 					destEncoding;
	UInt32 							reserved1;
	UInt32 							reserved2;
	TECBufferContextRec 			bufferContext;
																/* private - manipulated only within Plugin*/
	UInt32 							contextRefCon;
	ProcPtr 						conversionProc;
	ProcPtr 						flushProc;
	ProcPtr 						clearContextInfoProc;
	UInt32 							options1;
	UInt32 							options2;
	TECPluginStateRec 				pluginState;

};
typedef struct TECConverterContextRec TECConverterContextRec;

struct TECSnifferContextRec {

																/* public - manipulated externally*/
	Ptr 							pluginRec;
	TextEncoding 					encoding;
	ItemCount 						maxErrors;
	ItemCount 						maxFeatures;
	TextPtr 						textInputBuffer;
	TextPtr 						textInputBufferEnd;
	ItemCount 						numFeatures;
	ItemCount 						numErrors;
																/* private - manipulated only within Plugin*/
	UInt32 							contextRefCon;
	ProcPtr 						sniffProc;
	ProcPtr 						clearContextInfoProc;
	TECPluginStateRec 				pluginState;
};
typedef struct TECSnifferContextRec TECSnifferContextRec;

/*
####################################################################################
		Functional Messages
####################################################################################
*/
typedef OSStatus (*TECPluginNewEncodingConverterPtr)(TECObjectRef *newEncodingConverter, TECConverterContextRec *plugContext, TextEncoding inputEncoding, TextEncoding outputEncoding);
typedef OSStatus (*TECPluginClearContextInfoPtr)(TECObjectRef encodingConverter, TECConverterContextRec *plugContext);
typedef OSStatus (*TECPluginConvertTextEncodingPtr)(TECObjectRef encodingConverter, TECConverterContextRec *plugContext);
typedef OSStatus (*TECPluginFlushConversionPtr)(TECObjectRef encodingConverter, TECConverterContextRec *plugContext);
typedef OSStatus (*TECPluginDisposeEncodingConverterPtr)(TECObjectRef newEncodingConverter, TECConverterContextRec *plugContext);
typedef OSStatus (*TECPluginNewEncodingSnifferPtr)(TECSnifferObjectRef *encodingSniffer, TECSnifferContextRec *snifContext, TextEncoding inputEncoding);
typedef OSStatus (*TECPluginClearSnifferContextInfoPtr)(TECSnifferObjectRef encodingSniffer, TECSnifferContextRec *snifContext);
typedef OSStatus (*TECPluginSniffTextEncodingPtr)(TECSnifferObjectRef encodingSniffer, TECSnifferContextRec *snifContext);
typedef OSStatus (*TECPluginDisposeEncodingSnifferPtr)(TECSnifferObjectRef encodingSniffer, TECSnifferContextRec *snifContext);
typedef OSStatus (*TECPluginGetCountAvailableTextEncodingsPtr)(TextEncoding *availableEncodings, ItemCount maxAvailableEncodings, ItemCount *actualAvailableEncodings);
typedef OSStatus (*TECPluginGetCountAvailableTextEncodingPairsPtr)(TECConversionInfo *availableEncodings, ItemCount maxAvailableEncodings, ItemCount *actualAvailableEncodings);
typedef OSStatus (*TECPluginGetCountDestinationTextEncodingsPtr)(TextEncoding inputEncoding, TextEncoding *destinationEncodings, ItemCount maxDestinationEncodings, ItemCount *actualDestinationEncodings);
typedef OSStatus (*TECPluginGetCountSubTextEncodingsPtr)(TextEncoding inputEncoding, TextEncoding subEncodings[], ItemCount maxSubEncodings, ItemCount *actualSubEncodings);
typedef OSStatus (*TECPluginGetCountAvailableSniffersPtr)(TextEncoding *availableEncodings, ItemCount maxAvailableEncodings, ItemCount *actualAvailableEncodings);
typedef OSStatus (*TECPluginGetTextEncodingInternetNamePtr)(TextEncoding textEncoding, Str255 encodingName);
typedef OSStatus (*TECPluginGetTextEncodingFromInternetNamePtr)(TextEncoding *textEncoding, ConstStr255Param encodingName);
typedef OSStatus (*TECPluginGetCountWebEncodingsPtr)(TextEncoding *availableEncodings, ItemCount maxAvailableEncodings, ItemCount *actualAvailableEncodings);
typedef OSStatus (*TECPluginGetCountMailEncodingsPtr)(TextEncoding *availableEncodings, ItemCount maxAvailableEncodings, ItemCount *actualAvailableEncodings);
/*
####################################################################################
		Dispatch Table Definition
####################################################################################
*/

enum {
	kTECPluginDispatchTableVersion1 = 0x00010000,				/* 1.0 through 1.0.3 releases*/
	kTECPluginDispatchTableVersion1_1 = 0x00010001,				/* 1.1 releases*/
	kTECPluginDispatchTableVersion1_2 = 0x00010002,				/* 1.2 releases*/
	kTECPluginDispatchTableCurrentVersion = kTECPluginDispatchTableVersion1_2
};

struct TECPluginDispatchTable {

	TECPluginVersion 				version;
	TECPluginVersion 				compatibleVersion;
	TECPluginSignature 				PluginID;

	TECPluginNewEncodingConverterPtr  PluginNewEncodingConverter;
	TECPluginClearContextInfoPtr 	PluginClearContextInfo;
	TECPluginConvertTextEncodingPtr  PluginConvertTextEncoding;
	TECPluginFlushConversionPtr 	PluginFlushConversion;
	TECPluginDisposeEncodingConverterPtr  PluginDisposeEncodingConverter;

	TECPluginNewEncodingSnifferPtr 	PluginNewEncodingSniffer;
	TECPluginClearSnifferContextInfoPtr  PluginClearSnifferContextInfo;
	TECPluginSniffTextEncodingPtr 	PluginSniffTextEncoding;
	TECPluginDisposeEncodingSnifferPtr  PluginDisposeEncodingSniffer;

	TECPluginGetCountAvailableTextEncodingsPtr  PluginGetCountAvailableTextEncodings;
	TECPluginGetCountAvailableTextEncodingPairsPtr  PluginGetCountAvailableTextEncodingPairs;
	TECPluginGetCountDestinationTextEncodingsPtr  PluginGetCountDestinationTextEncodings;
	TECPluginGetCountSubTextEncodingsPtr  PluginGetCountSubTextEncodings;
	TECPluginGetCountAvailableSniffersPtr  PluginGetCountAvailableSniffers;
	TECPluginGetCountWebEncodingsPtr  PluginGetCountWebTextEncodings;
	TECPluginGetCountMailEncodingsPtr  PluginGetCountMailTextEncodings;

	TECPluginGetTextEncodingInternetNamePtr  PluginGetTextEncodingInternetName;
	TECPluginGetTextEncodingFromInternetNamePtr  PluginGetTextEncodingFromInternetName;

};
typedef struct TECPluginDispatchTable TECPluginDispatchTable;


#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif

#if PRAGMA_IMPORT_SUPPORTED
#pragma import off
#endif

#ifdef __cplusplus
}
#endif

#endif /* __TEXTENCODINGPLUGIN__ */

